#include "cv.h"
#include "highgui.h"
#include <math.h>




int* cvHist(IplImage *Obr, int visible);
//vraci ukazatel na pole velikosti 256, kde kazdy prvek obsahuje cetnost daneho jasu v obraze
//provede vypocet histogramu z obrazu Obr
//promenna visible udava jestli se ma zobrazovat histogram =1 ci nikoliv =0

IplImage* cvImageAdjust(IplImage *Obr,double x1,double y1,double x2,double y2);
//provede zmenu jasu obrazu
//parametry jsou obraz, vysledek, souradnice primky, podle niz jas menime

IplImage* cvImageRotate(IplImage* Obr,int uhel,int orez);
//proved rotaci obrazu Obr o uhel uhel
//jestlize orez=0 pak bude vyseldny obraz vetsi nez vzorovy ovsem cely, jinak se obraz orizne
//na velikost originalu

IplImage* cvHitOrMiss(IplImage* Obr,int* a, int x, int y);
//provede operaci hit or miss s elementem a (o velikosti x,y, stred je vzdy uprostred elementu) na obraz Obr
//vraci vzsledny obraz 

IplImage* cvSkel(IplImage* Obr);
//provede vypocet skeletu daneho obrazu pomoci metody zalozene na vzdalenostni mape

int main( int argc, char** argv )
{
	IplImage *obr;
	IplImage *vys;
	IplImage *gray;
	IplImage *bw;
	IplImage *skel;
	IplConvKernel *b;//strukturni element
	int a[] ={1,0,-1,
			  1,0,-1,
			  1,0,-1};//struturni element pro operaci hitormis

	int Hodnota=200;//nastaveni pocatecni hodnoty posuvniku
	int c;

	cvNamedWindow( "Okno",1);//vytvori okno na ktere se odkazujeme pomoci jmena
	
	cvCreateTrackbar( "Posuvnik", "Okno", &Hodnota, 1000, 0 );//vztvoreni posuvniku jeho hodnota se uklada do promenne int


	obr = cvLoadImage ( "head.bmp",-1);//nacte obraz
	vys = cvCreateImage (cvGetSize(obr),8,1);
	gray = cvCreateImage ( cvGetSize(obr), 8, 1);// vztvoreni hlavycky obrazu a alokovani pameti 
	cvCvtColor( obr, gray, CV_BGR2GRAY);

	
	printf("**stiskni klavesu**\n");
	
	cvShowImage("Okno",gray);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund
    
	
	b= cvCreateStructuringElementEx( 3, 3, 1, 1, CV_SHAPE_RECT, NULL );//vytvoreni strukturniho elementu
	//vytvori struturni element, parametry jsou :velikost x,y, stred elementu x,y, tvar elementu, pripadna data elementu
	cvErode(gray , vys, b, 1 );//provede jednu erozi obrazu gray pomoci elemenu b
	printf("sedotonova eroze\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",vys);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund
    
	cvDilate(gray , vys, b, 1 );//provede jednu dilataci obrazu gray pomoci elemenu b
	printf("sedotonova dilatace\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",vys);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund

	cvMorphologyEx(gray, vys, NULL, b, CV_MOP_OPEN , 1);//provede jednu operaci otevreni
	printf("sedotonove otevreni\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",vys);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund


	cvMorphologyEx(gray, vys, NULL, b, CV_MOP_CLOSE , 1);//provede jednu operaci otevreni
    printf("sedotonova uzavreni\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",vys);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund

	bw = cvHitOrMiss( gray,a, 3, 3);
	printf("hit or miss transformace\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",bw);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund

	
	skel = cvSkel(gray);//vypocet skeletu
	printf("vypocet skeletu\n");
	printf("**stiskni klavesu**\n");
	cvShowImage("Okno",skel);//zobrazi obraz do predem vytvoreneho okna
	c = cvWaitKey(0);//cekani na klavesu v milisekund

    cvDestroyWindow("Okno");//uvolneni okna    
	cvReleaseImage(&obr);//uvolneni pameti
	cvReleaseImage(&skel);//uvolneni pameti
	cvReleaseImage(&bw);//uvolneni pameti
	cvReleaseImage(&vys);//uvolneni pameti
	cvReleaseImage(&gray);//uvolneni pameti



    return 0;
}




int* cvHist(IplImage *Obr, int visible)
//vypocet histogramu
{
	int i,j,max=0;
	int *Hist;//pole pro umisteni hodnot histogramu
	IplImage *Histogram;
	int range=256;//pocet jasu
	int c;

	//alokace pameti pro histogram a jeho vynulovani
	Hist = (int*) malloc(sizeof(int)*range);
	for (i=0;i<range;i++){
		Hist[i]=0;//vynulovani histgramu
	}


	for (i=0;i<Obr->height;i++){
		for (j=0;j<Obr->width;j++){
			Hist[(unsigned char)Obr->imageData[j+i*Obr->width]]++;//inkrementace prislusneho jasu
		}
	}

	if (visible==1){
		cvNamedWindow( "Histogram", 1 );
		for (i=0;i<=range;i++){
			if (max<Hist[i]){
				max=Hist[i];
			}
		}
		Histogram= cvCreateImage( cvSize(range,300),8,1);//vytvori obraz pro zobrazeni histogramu 300 cely histogram se musi max/300
		for (i=0;i<=range;i++){
			cvLine(Histogram,cvPoint(i,(int)(300-Hist[i]/((float)max/300))),cvPoint(i,300),cvRealScalar(255),0,8,0);
			cvLine(Histogram,cvPoint(i,(int)(300-Hist[i]/((float)max/300))),cvPoint(i,0),cvRealScalar(0),0,8,0);
		}
		cvShowImage( "Histogram", Histogram);
		c = cvWaitKey(0);//cekani na klavesu v milisekundach
		cvDestroyWindow("Histogram");
		cvReleaseImage(&Histogram);
	}
	return(Hist);
}

IplImage* cvImageAdjust(IplImage *Obr, double x1,double y1,double x2,double y2)
{
	double k,q;//parametry primky
	int pole[256];//prepocet hodnot
	int i,j;
	IplImage *Vys;

	Vys = cvCreateImage( cvGetSize(Obr),8,Obr->nChannels);
	k=(y2-y1)/(x2-x1);
	q=y1-k*x1;
	for (i=0;i<256;i++){
		if (k*i/256+q<0){
			pole[i]=(unsigned char)0;
		}
		else{
			if (k*i/256+q>1){
				pole[i]=(unsigned char)255;
			}
			else{
				pole[i]=(unsigned char)(1+(k*i/256+q)*255);
			}
		}
	}
	for (i=0;i<Obr->widthStep;i++){
		for (j=0;j<Obr->height;j++){
			Vys->imageData[j*Obr->widthStep+i]=pole[(unsigned char)Obr->imageData[j*Obr->widthStep+i]];
		}
	}
	return (Vys);
}

IplImage* cvImageRotate(IplImage* Obr,int uhel,int orez)
{
	float m[6];
    CvMat M = cvMat( 2, 3, CV_32F, m );
    double prep;
	int vyska;
	int sirka;
	double uhelpom;
	IplImage *Vys;
	 
	
	if (orez==0){//obraz se zvetsi
		uhelpom=atan((double)(Obr->height/2)/(double)(Obr->width/2));//vypocita se pomocny uhel 
		prep=sqrt(Obr->width*Obr->width+Obr->height*Obr->height)/2;
		if (sin(uhelpom+uhel*CV_PI/180.)*prep>sin(uhelpom-uhel*CV_PI/180.)*prep){
			vyska=sin(uhelpom+uhel*CV_PI/180.)*prep;
		}
		else{
			vyska=sin(uhelpom-uhel*CV_PI/180.)*prep;
		}
		if (cos(uhelpom+uhel*CV_PI/180.)*prep>cos(uhelpom-uhel*CV_PI/180.)*prep){
			sirka=cos(uhelpom+uhel*CV_PI/180.)*prep;
		}
		else{
			sirka=cos(uhelpom-uhel*CV_PI/180.)*prep;
		}
		Vys= cvCreateImage( cvSize(sirka*2,vyska*2),8,Obr->nChannels);//vytvoreni vysledneho obrazu
	}
	else{
		Vys= cvCloneImage( Obr);//vytvoreni vysledneho obrazu
	}
	m[0] = (float)(cos(-uhel*CV_PI/180.));
    m[1] = (float)(sin(-uhel*CV_PI/180.));
    m[2] = Obr->width*0.5f;
    m[3] = -m[1];
    m[4] = m[0];
    m[5] = Obr->height*0.5f;

	

    cvGetQuadrangleSubPix( Obr, Vys, &M, 1, cvScalarAll(0));//provede se rotace
	
    return(Vys);
}


IplImage* cvHitOrMiss(IplImage* Obr,int* a,int x,int y){
	
	int i,j,k,l;
	IplImage* Vys;
	int vlajka;

	Vys = cvCreateImage(cvGetSize(Obr),8,1);//vytvoreni vystupniho obrazu
	for (i=0;i<Obr->width;i++){//prochazi obraz
		for (j=0;j<Obr->height;j++){
				Vys->imageData[i+j*Obr->widthStep]=0;
		}
	}
	for (i=0+x/2;i<Obr->width-x/2;i++){//prochazi obraz
		for (j=0+y/2;j<Obr->height-y/2;j++){
			vlajka=1;
			for (k=0;k<x;k++){//prochzi element
				for (l=0;l<y;l++){
					if (a[k+l*x]==-1 && Obr->imageData[i+(k-x/2)+(j+(l-y/2))*Obr->widthStep]!=0){
						vlajka=0;
					}
					if (a[k+l*x]==1 && Obr->imageData[i+(k-x/2)+(j+(l-y/2))*Obr->widthStep]==0){
						vlajka=0;
					}
				}
			}
			if (vlajka==1){
				Vys->imageData[i+j*Obr->widthStep]=255;
			}
			else{
				Vys->imageData[i+j*Obr->widthStep]=0;
			}
		}
	}
	return(Vys);
}
			
IplImage* cvSkel(IplImage* gray){


    int i,j;
	IplImage* dist = 0;
	IplImage* Dist = 0;
	IplImage* Vys = 0;
	IplImage* edge = 0;


    //gray = cvLoadImage( "1.bmp", 0 );


    // Create the output image
    dist = cvCreateImage( cvSize(gray->width,gray->height), IPL_DEPTH_32F, 1 );
    Dist = cvCloneImage( gray );
    Vys = cvCloneImage( gray );
    
    // Convert to grayscale
    edge = cvCloneImage( gray );

    // Create a window
    

	for (i=0;i<Vys->height;i++){
		for (j=0;j<Vys->width;j++){
			Vys->imageData[i*Vys->widthStep+j]=0;
		}
	}

    
    
    cvThreshold( gray, edge, 100, 100, CV_THRESH_BINARY );
    //Distance transform                  
    cvDistTransform( edge, dist, CV_DIST_L1 , 3, NULL );

    cvConvertScale( dist, Dist, 1, 0 );
    for (i=1;i<Dist->height-2;i++){
		for (j=1;j<Dist->width-2;j++){
			if ((Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[(i-1)*Dist->widthStep+j] && Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[(i+1)*Dist->widthStep+j]) || (Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[(i-1)*Dist->widthStep+j] && Dist->imageData[i*Dist->widthStep+j]==Dist->imageData[(i+1)*Dist->widthStep+j] && Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[(i+2)*Dist->widthStep+j])){
				Vys->imageData[i*Vys->widthStep+j]=(char)255;
			}
			if ((Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[i*Dist->widthStep+j-1] && Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[i*Dist->widthStep+j+1]) || (Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[i*Dist->widthStep+j-1] && Dist->imageData[i*Dist->widthStep+j]==Dist->imageData[i*Dist->widthStep+j+1] && Dist->imageData[i*Dist->widthStep+j]>Dist->imageData[i*Dist->widthStep+j+2])){
				Vys->imageData[i*Vys->widthStep+j]=(char)255;
			}
		}
	}


	
	cvReleaseImage( &gray );
    cvReleaseImage( &edge );
    cvReleaseImage( &dist );
    cvReleaseImage( &Dist );
	return(Vys);
}
